﻿using Microsoft.BizTalk.Message.Interop;
using Microsoft.BizTalk.Component.Interop;
using System;
using System.Collections;
using System.IO;
using Ionic.Zip;

namespace BizTalkLive.UnZip
{
    //Attributes of class 
    [ComponentCategory(CategoryTypes.CATID_PipelineComponent)]
    [ComponentCategory(CategoryTypes.CATID_DisassemblingParser)]

    //Generate unique identifier
    [System.Runtime.InteropServices.Guid("F7EA6794-5E97-4E20-9C4B-EA9C3FD10FA9")]
    public class UnZip : IBaseComponent, IComponentUI, IDisassemblerComponent, IPersistPropertyBag
    {
        private string _ShemaMessageType;
        private string _PropertyNamespace;
        public string ShemaMessageType
        {
            get
            {
                return _ShemaMessageType;
            }
            set
            {
                _ShemaMessageType = value;
            }
        }
        public string PropertyNamespace
        {
            get
            {
                return _PropertyNamespace;
            }
            set
            {
                _PropertyNamespace = value;
            }
        }

        #region IBaseComponent
        public string Description
        {
            get
            {
                return "Pipeline component used to unzip a message";
            }
        }
        public string Name
        {
            get
            {
                return "UnZip_Disassembler";
            }
        }
        public string Version
        {
            get
            {
                return "1.0.0.0";
            }
        }
        #endregion

        #region IPersistPropertyBag
        /// <summary>
        /// Gets class ID of component for usage from unmanaged code.
        /// </summary>
        /// <param name="classid">
        /// Class ID of the component
        /// </param>
        public void GetClassID(out System.Guid classid)
        {
            classid = new System.Guid("F7EA6794-5E97-4E20-9C4B-EA9C3FD10FA9");
        }

        /// <summary>
        /// not implemented
        /// </summary>
        public void InitNew()
        {
        }

        /// <summary>
        /// Loads configuration properties for the component
        /// </summary>
        /// <param name="pb">Configuration property bag</param>
        /// <param name="errlog">Error status</param>
        public virtual void Load(Microsoft.BizTalk.Component.Interop.IPropertyBag pb, int errlog)
        {
            object val = null;
            val = this.ReadPropertyBag(pb, "ShemaMessageType");
            if ((val != null))
            {
                this.ShemaMessageType = ((string)(val));
            }
            val = this.ReadPropertyBag(pb, "PropertyNamespace");
            if ((val != null))
            {
                this.PropertyNamespace = ((string)(val));
            }
        }
        //public virtual void Load(Microsoft.BizTalk.Component.Interop.IPropertyBag pb, int errlog)
        //{
        //    object val = null;
        //    val = this.ReadPropertyBag(pb, "PropertyNamespace");
        //    if ((val != null))
        //    {
        //        this._PropertyNamespace = ((string)(val));
        //    }
        //}

        /// <summary>
        /// Saves the current component configuration into the property bag
        /// </summary>
        /// <param name="pb">Configuration property bag</param>
        /// <param name="fClearDirty">not used</param>
        /// <param name="fSaveAllProperties">not used</param>
        public virtual void Save(Microsoft.BizTalk.Component.Interop.IPropertyBag pb, bool fClearDirty, bool fSaveAllProperties)
        {
            this.WritePropertyBag(pb, "ShemaMessageType", this.ShemaMessageType);
            this.WritePropertyBag(pb, "PropertyNamespace", this.PropertyNamespace);
        }

        #region utility functionality
        /// <summary>
        /// Reads property value from property bag
        /// </summary>
        /// <param name="pb">Property bag</param>
        /// <param name="propName">Name of property</param>
        /// <returns>Value of the property</returns>
        private object ReadPropertyBag(Microsoft.BizTalk.Component.Interop.IPropertyBag pb, string propName)
        {
            object val = null;
            try
            {
                pb.Read(propName, out val, 0);
            }
            catch (System.ArgumentException)
            {
                return val;
            }
            catch (System.Exception e)
            {
                throw new System.ApplicationException(e.Message);
            }
            return val;
        }

        /// <summary>
        /// Writes property values into a property bag.
        /// </summary>
        /// <param name="pb">Property bag.</param>
        /// <param name="propName">Name of property.</param>
        /// <param name="val">Value of property.</param>
        private void WritePropertyBag(Microsoft.BizTalk.Component.Interop.IPropertyBag pb, string propName, object val)
        {
            try
            {
                pb.Write(propName, ref val);
            }
            catch (System.Exception e)
            {
                throw new System.ApplicationException(e.Message);
            }
        }
        #endregion

        #endregion

        #region IComponentUI
        public IntPtr Icon
        {
            get
            {
                return IntPtr.Zero;
            }
        }
        public IEnumerator Validate(object projectystem)
        {
            return null;
        }
        #endregion

        #region IDisassemblerComponent

        private System.Collections.Queue OutFiles = new System.Collections.Queue();
        public void Disassemble(IPipelineContext pc, IBaseMessage inmsg)
        {
            //Microsoft.BizTalk.CAT.BestPractices.Framework.Instrumentation.TraceManager.WorkflowComponent.TraceInfo("TSAUnZipDecode : Unzip process has started at " + DateTime.Now.ToLongDateString());
            Byte[] TextFileBytes = null;
            string FileStatus = "0";
            string extension = "";
            string strReceivePortName = "";
            string strReceivedFileName = "";
            string _filename = "";

            IBaseMessagePart msgBodyPart1 = inmsg.BodyPart;

            Stream msgBodyPartStream1 = msgBodyPart1.GetOriginalDataStream();

            MemoryStream memStream1 = new MemoryStream();
            byte[] buffer1 = new Byte[1024];

            int bytesRead1 = 1024;
            while (bytesRead1 != 0)
            {
                bytesRead1 = msgBodyPartStream1.Read(buffer1, 0, buffer1.Length);
                memStream1.Write(buffer1, 0, bytesRead1);
            }

            IBaseMessage outMessage1;
            outMessage1 = pc.GetMessageFactory().CreateMessage();
            outMessage1.AddPart("Body", pc.GetMessageFactory().CreateMessagePart(), true);
            memStream1.Position = 0L;
            outMessage1.BodyPart.Data = memStream1;
            if (this.ShemaMessageType != null)
            {
                outMessage1.Context.Promote("MessageType", "http://schemas.microsoft.com/BizTalk/2003/system-properties", this.ShemaMessageType);

            }
            outMessage1.Context.Promote("FileExtension", PropertyNamespace, ".zip");

            IBaseMessageContext context = inmsg.Context;
            strReceivePortName = context.Read("ReceivePortName", "http://schemas.microsoft.com/BizTalk/2003/system-properties").ToString();
            strReceivedFileName = context.Read("ReceivedFileName", "http://schemas.microsoft.com/BizTalk/2003/file-properties").ToString();
            strReceivedFileName = strReceivedFileName.Substring(strReceivedFileName.LastIndexOf("\\") + 1);
            strReceivedFileName = ((string)strReceivedFileName).Replace(".zip", "");
            outMessage1.Context.Promote("ReceivedFileName", "http://schemas.microsoft.com/BizTalk/2003/file-properties", strReceivedFileName);
            outMessage1.Context.Promote("ReceivePortName", "http://schemas.microsoft.com/BizTalk/2003/system-properties", strReceivePortName);

            //outMessage1.Context = PipelineUtil.CloneMessageContext(inmsg.Context);

            OutFiles.Enqueue(outMessage1);

            IBaseMessagePart msgBodyPart_XML = inmsg.BodyPart;

            IBaseMessage outMessage;

            if (msgBodyPart_XML != null)
            {
                Stream msgBodyPartStream = msgBodyPart_XML.GetOriginalDataStream();
                msgBodyPartStream.Position = 0L;

                if (msgBodyPartStream != null)
                {
                    using (ZipInputStream zipInputStream = new ZipInputStream(msgBodyPartStream))
                    {
                        ZipEntry entry = zipInputStream.GetNextEntry();
                        while (entry != null)
                        {
                            Stream memStream = new MemoryStream();
                            byte[] buffer = new Byte[4096];

                            int bytesRead = 4096;
                            while (bytesRead != 0)
                            {
                                bytesRead = zipInputStream.Read(buffer, 0, buffer.Length);
                                memStream.Write(buffer, 0, bytesRead);
                            }

                            //Creating custom context property to hold extension of file
                            extension = entry.FileName.Substring(entry.FileName.IndexOf(".")).ToUpper();
                            _filename = entry.FileName.ToString();

                            if (extension == ".XML")
                            {
                                FileStatus = "1";
                                outMessage = pc.GetMessageFactory().CreateMessage();
                                outMessage.AddPart("Body", pc.GetMessageFactory().CreateMessagePart(), true);
                                memStream.Position = 0L;
                                outMessage.BodyPart.Data = memStream;

                                outMessage.Context = PipelineUtil.CloneMessageContext(inmsg.Context);
                                if (this.ShemaMessageType != null)
                                {
                                    outMessage.Context.Promote("MessageType", "http://schemas.microsoft.com/BizTalk/2003/system-properties", this.ShemaMessageType);

                                }
                                _filename = ((string)_filename).Replace(".xml", "");
                                outMessage.Context.Promote("ReceivedFileName", "http://schemas.microsoft.com/BizTalk/2003/file-properties", _filename);
                                outMessage.Context.Promote("FileStatus", PropertyNamespace, FileStatus.ToString());
                                outMessage.Context.Promote("FileExtension", PropertyNamespace, extension.ToLower());

                                OutFiles.Enqueue(outMessage);

                            }
                            else
                            {
                                FileStatus = "0";
                                //outMessage.Context.Promote("FileStatus", PropertyNamespace, FileStatus.ToString());
                                //outMessage.Context.Promote("FileExtension", PropertyNamespace, extension.ToLower());
                            }

                            entry = zipInputStream.GetNextEntry();
                        }

                        //Microsoft.BizTalk.CAT.BestPractices.Framework.Instrumentation.TraceManager.WorkflowComponent.TraceInfo("TSAUnZipDecode : FileStatus: {0} and SignStatus: {1}  Unzip process has successfully completed at " + DateTime.Now.ToLongTimeString(), FileStatus.ToString(), SignStatus.ToString());
                    }

                }

            }
        }

        public IBaseMessage GetNext(IPipelineContext pContext)
        {
            if (OutFiles.Count > 0)
            {
                return (IBaseMessage)OutFiles.Dequeue();
            }
            else
            {
                return null;
            }

        }

        #endregion
    }
}
